@@ -5,8 +5,9 @@ use menmos_client::{Meta, Query, Type};
 
 use snafu::prelude::*;
 
-use crate::{ClientRC, FileMetadata, Result};
+use crate::{ClientRC, FileMetadata};
 
+use super::error::*;
 use super::file::MenmosFile;
 use crate::util;
 
@@ -43,21 +44,24 @@ impl MenmosDirectory {
         let blob_id = client
             .create_empty(metadata)
             .await
-            .with_whatever_context(|e| format!("failed to create directory: {e}"))?;
+            .map_err(|_| FsError::DirCreateError)?;
 
         Ok(Self { blob_id, client })
     }
 
     #[doc(hidden)]
     pub async fn open(client: ClientRC, id: &str) -> Result<Self> {
-        let metadata = util::get_meta(&client, id).await?;
+        let metadata = util::get_meta(&client, id).await.context(DirOpenSnafu)?;
         Self::open_raw(client, id, metadata)
     }
 
     pub(crate) fn open_raw(client: ClientRC, id: &str, meta: BlobMeta) -> Result<Self> {
-        if meta.blob_type == Type::File {
-            whatever!("is file");
-        }
+        ensure!(
+            meta.blob_type == Type::Directory,
+            ExpectedDirectorySnafu {
+                blob_id: String::from(id)
+            }
+        );
 
         Ok(Self {
             blob_id: String::from(id),
@@ -71,24 +75,30 @@ impl MenmosDirectory {
     }
 
     /// Get a stream of entries present in this directory.
-    pub fn list(&self) -> impl TryStream<Ok = DirEntry, Error = snafu::Whatever> + Unpin {
+    pub fn list(&self) -> impl TryStream<Ok = DirEntry, Error = FsError> + Unpin {
         let query = Query::default()
             .and_parent(&self.blob_id)
             .with_from(0)
             .with_size(50);
 
         let client = self.client.clone();
-        Box::pin(util::scroll_query(query, &client).and_then(move |hit| {
-            let client = client.clone();
-            async move {
-                let entry = if hit.meta.blob_type == Type::File {
-                    DirEntry::File(MenmosFile::open_raw(client, &hit.id, hit.meta)?)
-                } else {
-                    DirEntry::Directory(MenmosDirectory::open_raw(client, &hit.id, hit.meta)?)
-                };
-                Ok(entry)
-            }
-        }))
+        Box::pin(
+            util::scroll_query(query, &client)
+                .map_err(|source| FsError::DirQueryError { source })
+                .and_then(move |hit| {
+                    let client = client.clone();
+                    async move {
+                        let entry = if hit.meta.blob_type == Type::File {
+                            DirEntry::File(MenmosFile::open_raw(client, &hit.id, hit.meta)?)
+                        } else {
+                            DirEntry::Directory(MenmosDirectory::open_raw(
+                                client, &hit.id, hit.meta,
+                            )?)
+                        };
+                        Ok(entry)
+                    }
+                }),
+        )
     }
 
     /// Get whether this directory has any children.
@@ -98,7 +108,7 @@ impl MenmosDirectory {
             .client
             .query(query)
             .await
-            .with_whatever_context(|e| format!("failed to query: {e}"))?;
+            .map_err(|_| FsError::DirListError)?;
 
         Ok(results.total == 0)
     }
