@@ -1,4 +1,5 @@
 use std::collections::HashMap;
+use std::path::PathBuf;
 use std::sync::Arc;
 
 use menmos_client::Client;
@@ -56,3 +57,18 @@ impl FileMetadata {
         self
     }
 }
+
+#[derive(Debug, Clone, PartialEq)]
+pub struct UploadRequest {
+    /// The path of the file to upload.
+    pub path: PathBuf,
+
+    /// The metadata of the file to upload.
+    pub metadata: HashMap<String, String>,
+
+    /// The tags of the file to upload.
+    pub tags: Vec<String>,
+
+    /// The parent id of the file to upload.
+    pub parent_id: Option<String>,
+}
